/* file: data/static/games/four_in_a_row/four_in_a_row.css */
.fiar-board {
    border-collapse: collapse;
    margin-top: 1em;
}
.fiar-board td {
    width: 40px;
    height: 40px;
    border: 1px solid #555;
    background: #e0e0ff;
}
.fiar-board td.player1 { background: #ff4c4c; }
.fiar-board td.player2 { background: #ffd700; }
.fiar-column button {
    width: 100%;
    height: 32px;
}
